<?php

namespace App\Http\Controllers;

use App\Helpers\Plates;
use App\Http\Helpers\DVLAClient;
use App\Http\Helpers\VRN;
use App\Models\Mark;
use Carbon\Carbon;
use Eloquenty\Entries\EntryModel;
use Illuminate\Http\Request;
use Illuminate\Http\Response;
use Illuminate\Support\Facades\Cookie;
use Illuminate\Support\Facades\Log;
use Statamic\Facades\Entry;
use Stripe\Stripe;
use Illuminate\Support\Facades\Mail;

class EcommerceController extends Controller
{
    //
    public function getCartStatus(Request $request)
    {
        $value = $request->cookie('cartKey');
        if ($value){
            $entry=Entry::findBySlug($value,"orders");
            if ($entry){
                Mark::updateCart($entry);
                $items=$entry->get("items");
                if (!$items) $items=[];
                return ["active"=>(count($items)>0)];
            }

        }
        return ["active"=>false];
    }

    public function createIntent(Request $request)
    {

        $cart = $request->cookie('cartKey');
        if ($cart){
           
            $entry=Entry::findBySlug($cart,"orders");
            if ($entry) {
                $amount = $entry->get("grand_total") * 100;
                Stripe::setApiKey(config("services.stripe.private_key"));
                header('Content-Type: application/json');
                try {
                    $paymentIntent = \Stripe\PaymentIntent::create([
                        'amount' => $amount,
                        'currency' => 'gbp',
                    ]);
                    $output = [
                        'clientSecret' => $paymentIntent->client_secret,
                    ];
                    echo json_encode($output);
                    return;
                } catch (\Exception $e) {
                    http_response_code(500);
                    echo json_encode(['error' => $e->getMessage()]);
                    return;
                }
            }
        }
        http_response_code(500);
        echo json_encode(['error' => "No cart is available"]);
        return;
    }
    
    public function createFirstIntent(Request $request)
    {
        $cart = $request->cookie('cartKey');
        if ($cart){
            $entry=Entry::findBySlug($cart,"orders");
            if ($entry) {
                $amount = $entry->get("grand_total") * 100;
                Stripe::setApiKey(config("services.stripe.private_key"));
                header('Content-Type: application/json');
                try {
                    // $paymentIntent = \Stripe\PaymentIntent::create([
                    //     'amount' => $amount,
                    //     'currency' => 'gbp',
                    // ]);
                    $output = [
                        // $paymentIntent->client_secret
                        'clientSecret' => 'abcdedfeef',
                        'intent_id' => '23342323232323',
                    ];
                    echo json_encode($output);
                    return;
                } catch (\Exception $e) {
                    http_response_code(500);
                    echo json_encode(['error' => $e->getMessage()]);
                    return;
                }
            }
        }
        http_response_code(500);
        echo json_encode(['error' => "No cart is available"]);
        return;
    }


    public function addToCart(Request $request)
    {
        
        if (!$request->get("slug")) return redirect("/");

        $productSlug=$request->get("slug");
        $value = $request->cookie('cartKey');
        $response = new Response();
        if (!$value) {
            $value="cart-".$request->session()->getId()."-".time()."-".rand(0,9999999);
        }

        $cookie=new Cookie;
        //$entry=Entry::whereCollection("orders")->where("session_id",$value)->first();
        $entry=Entry::findBySlug($value,"orders");
        if (!$entry) {
            $entry = Entry::make();
            $entry->collection("orders");
            $entry->slug($value);
            $entry->set("title", "Cart Content");
        }

        $product=EntryModel::where("slug",$productSlug)->first();
        //$product=Entry::findBySlug($productSlug,"products");
        $isDVLA=false;
        $spaced = strtoupper(str_replace("-"," ",$productSlug));
        $vrn = strtoupper(str_replace("-","",$productSlug));
        if (!$product) {
            $price=0;
            $vat=0;
            $dot=0;
            $dateIssued=null;
            $vrnObject=new VRN($vrn);
            $enqOnly=true;
            $total_price=0;
            if (!$vrnObject->getError()){
                $isDVLA=true;
                DVLAClient::Init("ACEREG87");
                $dateIssued=$vrnObject->getDateIssued();
                $response=DVLAClient::GetPrice($vrn);
                if ($response && is_object($response) && property_exists($response,"id")){
                    $helper=new Plates();
                    $price=$helper->getCustomisedPrice($response->price);
                    if (property_exists($response,"enquiryOnly"))
                        $enqOnly=$response->enquiryOnly;
                    $vat=round($price*(env("VATRATE")/100),2);
                    $dot=80;
                    $total_price=$price+$vat+$dot;
                }
                else $spaced="";
            }
            if (!$isDVLA) return redirect("/");
        }
        else{
            $price=(float)$product->db_price;
            $vat=(float)$product->db_vat;
            $dot=(float)$product->dot;
            $total_price=(float)$product->db_total_price;
        }

        $items=$entry->get("items");
        if (!$items) $items=[];

        $found=false;
        foreach($items as $item){
            if (isset($item["id"]) && $item["id"]===$productSlug) $found=true;
        }

        if (!$found){
            $item=[];
            $item["id"]=$productSlug;
            $item["registration"]=$spaced;
            $item["base_price"]=$price;
            $item["include_plates"]=false;
            $item["plate_price_ex_vat"]=0;
            $item["vat"]=$vat;
            $item["dot_fee"]=$dot;
            $item["total"]=$total_price;
            $items[]=$item;
            $entry->set("items",$items);
            $entry->save();
        }
        Mark::updateCart($entry);
        return redirect('/cart')->withCookie('cartKey', $value);
    }
    
    public function requestValuation(Request $request)
    {
        Mail::send('email.valuation', ['data' => $request->all()], function ($message) {
                $message->to(config("app.enquiry_email"), config("app.enquiry_email_name"))
                ->subject('Request valuation');
        });
        $result = array('valuation'=>1);
        return json_encode($result);
    }
    
    public function enquirySubmit(Request $request)
    {
        Mail::send('email.enquiry-submit', ['data' => $request->all()], function ($message) {
                $message->to(config("app.enquiry_email"), config("app.enquiry_email_name"))
                ->subject('Number plate enquiry');
        });
        return redirect('/thank-you-for-enquiry');
    }
    
}
