<?php

namespace App\Http\Livewire;

use App\Mail\OrderCompleted;
use App\Mail\OrderNotified;
use App\Models\Mark;
use Illuminate\Support\Facades\Log;
use Illuminate\Support\Facades\Mail;
use Livewire\Component;
use Statamic\Facades\Entry;
use Statamic\Facades\GlobalSet;
use Stripe\Charge;
use Stripe\Customer;
use Stripe\Stripe;

class Cart extends Component
{
    public $cartKey;
    public $emptyCart=false;
    public $totalVat;
    public $grandTotal;
    public $stage=1;
    public $cardholder="";
    public $registered_keeper=[];
    public $issue_type=[];
    public $include_plates=[];
    public $current_reg=[];
    public $email="";
    public $billing_address1="";
    public $billing_address2="";
    public $billing_city="";
    public $billing_postcode="";
    public $telephone="";
    public $paymentIntent=null;
    public $terms = false;
    public $refreshSecret = false;

    protected $listeners = ["paymentIntentReceived"];

    protected $validationAttributes = [
        "email" => "email address",
        "cardholder" => "card holder",
        "billing_address1" => "first line of address",
        "billing_address2" => "second line of address",
        "billing_city" => "city or town",
        "billing_postcode" => "postcode",
/*        "registered_keeper.*" => "registered keeper",*/
        "issue_type.*" => "how to issue",
/*        "current_reg.*" => "current registration",*/
    ];
    protected $rules = [
        "email" => ["required","email"],
        "cardholder" => "required",
        "billing_address1" => "required",
        "billing_city" => "required",
        "billing_postcode" => "required",
/*        "registered_keeper.*" => ["required","min:5"],*/
        "issue_type.*" => ["required"]
    ];


    public function paymentIntentReceived($intent)
    {
        $entry=Entry::findBySlug($this->cartKey,"orders");
        if ($entry) {
            $entry->set("stripe_payment_reference",$intent);
            $entry->set("paid_date",date("Y-m-d H:i"));
            $entry->set("is_paid",true);
            $title=str_replace(" ","",$entry->get("title"));
            $orderId=(int)GlobalSet::findByHandle("settings")->inCurrentSite()->get("next_order_id");
            $set=GlobalSet::findByHandle("settings")->inCurrentSite();
            $set->set("next_order_id",$orderId+1);
            $set->save();
            $orderStr=(string)$orderId;
            $orderStr.="-".rand(9999,99999);
            $entry->set("order_id",$orderStr);
            $entry->slug("order_id",$orderStr);
            $entry->save();
            $items=$entry->items;
            Mail::to([$entry->get("billing_name")=> $entry->get("billing_email")])
                ->send(new OrderCompleted($entry->get("billing_name"),$items,$orderStr,$entry->grand_total));
            Mail::to([env("ORDER_NOTIFICATION_NAME")=> env("ORDER_NOTIFICATION_EMAIL")])
                ->send(new OrderNotified($entry->get("billing_name"),$items,$orderStr,$entry->grand_total));

            $this->stage = 3;
        }
    }
    public function updatedIncludePlates($field,$value)
    {
        $this->refreshSecret = true;
    }
    public function updated($params)
    {
        Log::info("update is run for updated");
        Log::info($params);
        $entry=Entry::findBySlug($this->cartKey,"orders");
        if ($entry) {
            $entry->set("billing_address1",$this->billing_address1);
            $entry->set("billing_address2",$this->billing_address2);
            $entry->set("billing_city",$this->billing_city);
            $entry->set("billing_postcode",$this->billing_postcode);
            $entry->set("billing_name",$this->cardholder);
            $entry->set("billing_email",$this->email);
            $entry->set("telephone",$this->telephone);
            $items=$entry->items;

            foreach($items as $index=>&$item){
                $key=$item["id"];
                $item["registered_keeper"]=$this->registered_keeper[$key]??"";
                if (isset($this->include_plates[$key]))
                    $item["include_plates"]=$this->include_plates[$key]=="1";
                else
                    $item["include_plates"]=false;
                $plateVat=0;
                if ($item["include_plates"]) {
                    $item["plate_price_ex_vat"] = (float)env("PLATE_PRICE");
                    $plateVat = round(((float)$item["plate_price_ex_vat"])*((float)env("VATRATE")/100),2);
                }
                else {
                    $item["plate_price_ex_vat"] = 0;
                }
                $item["total"] = $item["base_price"]+$item["vat"]+$item["dot_fee"]+$plateVat+$item["plate_price_ex_vat"];
                $item["issue_type"]=$this->issue_type[$key]??"";
                $item["current_reg"]=$this->current_reg[$key]??"";
                $items[$index]=$item;
            }
            $entry->set("items",$items);
            Log::info(print_r($items,true));
            $entry->save();
            Mark::updateCart($entry); // refresh the totals based on possibly changing the price with plates
            Log::info("Grand total before reload".$entry->get("grand_total"));
            $entry=Entry::findBySlug($this->cartKey,"orders"); // reload the entry to pick up the new totals
            Log::info("Grand total after reload".$entry->get("grand_total"));
        }

        $items=[];
        if (!$entry) {
            $this->emptyCart = true;
        }
        else {
            $items=$entry->get("items");
            $this->totalVat = $entry->get("tax_total");
            $this->grandTotal = $entry->get("grand_total");
        }

    }
    public function render()
    {
        $entry=Entry::findBySlug($this->cartKey,"orders");

        $items=[];
        if (!$entry) {
            $this->emptyCart = true;
        }
        else {
            Log::debug("Cart Render");
            Log::debug(print_r($entry,true));

            $items=$entry->get("items");
            $this->totalVat = $entry->get("tax_total");
            $this->grandTotal = $entry->get("grand_total");
        }
        // first load we dont want to do it but second and subsequent we do (gets changed in includeplates update
        if ($this->refreshSecret)
            // $this->dispatchBrowserEvent('getClientSecretAgain');
         $this->refreshSecret = false;
        return view('livewire.cart',["cartItems"=>$items]);
    }

    public function finalisePurchase()
    {
        $entry=Entry::findBySlug($this->cartKey,"orders");
        if ($entry) {
            $amount = $this->grandTotal * 100;
            $currency = 'gbp';
            $customer = null;
            Stripe::setApiKey(config("services.stripe.private_key"));
            try {
                $customer = Customer::create([
                    'email' => $this->email,
                    'source' => $this->stripe_token
                ]);
            } catch (\Exception $e) {
                $apiError = $e->getMessage();
            }

            if (empty($apiError) && $customer) {
                /** Charge a credit or a debit card */
                try {
                    /** Stripe charge class */
                    $charge = Charge::create(array(
                        'customer' => $customer->id,
                        'amount' => $amount,
                        'currency' => $currency,
                        'description' => print_r($entry->get("items"),true)
                    ));

                } catch (\Exception $e) {
                    $apiError = $e->getMessage();
                }

                if (empty($apiError) && $charge) {
                    // Retrieve charge details
                    $paymentDetails = $charge->jsonSerialize();
                    if ($paymentDetails['amount_refunded'] == 0 && empty($paymentDetails['failure_code']) && $paymentDetails['paid'] == 1 && $paymentDetails['captured'] == 1) {
                        /** You need to create model and other implementations */
                        /*
                        Payment::create([
                            'name'                          => request('name'),
                            'email'                         => request('email'),
                            'amount'                        => $paymentDetails['amount'] / 100,
                            'currency'                      => $paymentDetails['currency'],
                            'transaction_id'                => $paymentDetails['balance_transaction'],
                            'payment_status'                => $paymentDetails['status'],
                            'receipt_url'                   => $paymentDetails['receipt_url'],
                            'transaction_complete_details'  => json_encode($paymentDetails)
                        ]);
                        */
                        $this->stripe_token = "All worked. " . $paymentDetails["balance_transaction"];
                        //return redirect('/thankyou/?receipt_url=' . $paymentDetails['receipt_url']);
                    } else {
                        $this->stripe_token = $apiError;
                        session()->flash('error', 'Transaction failed');
                        return back()->withInput();
                    }
                } else {
                    $this->stripe_token = $apiError;
                    session()->flash('error', 'Error in capturing amount: ' . $apiError);
                    return back()->withInput();
                }
            } else {
                $this->stripe_token = $apiError;
                session()->flash('error', 'Invalid card details: ' . $apiError);
                return back()->withInput();
            }
        }
    }
    public function removeItem($id)
    {
        $entry=Entry::findBySlug($this->cartKey,"orders");
        if ($entry) {
            $items=$entry->get("items");
            if (!$items) $items=[];
            $found=false;
            foreach($items as $key=>$item){
                if (isset($item["id"]) && $item["id"]===$id) {
                    unset($items[$key]);
                    $found=true;
                }
            }
            if ($found){
                $entry->set("items",$items);
                $entry->save();
            }
        }
    }

    public function mount()
    {
        $entry=Entry::findBySlug($this->cartKey,"orders");
        if ($entry) {
            $this->cardholder = $entry->get("billing_name");
            $this->billing_address1 = $entry->get("billing_address1");
            $this->billing_address2 = $entry->get("billing_address2");
            $this->email = $entry->get("billing_email");
            $this->billing_city = $entry->get("billing_city");
            $this->billing_postcode = $entry->get("billing_postcode");
            $items=$entry->get("items");
            if (!$items) $items=[];
            foreach($items as $key=>$item){
                $this->registered_keeper[$item["id"]]=$item["registered_keeper"]??"";
            }
            //dd($this->registered_keeper);
           // dd($entry);
        }

    }
    public function placeOrder()
    {
        $this->validate();
        $this->dispatchBrowserEvent('generate-stripe-token');

    }
    public function orderDetails()
    {
        $this->stage = 2;
    }
    public function backToCart()
    {
        $this->stage = 1;
    }

}
