<?php

namespace App\Http\Livewire;

use App\Helpers\Plates;
use App\Http\Helpers\DVLAClient;
use Illuminate\Support\Facades\Log;
use Livewire\Component;
use Statamic\Facades\GlobalSet;

class Dvla extends Component
{
    public string $type;
    public string $alpha1;
    public string $alpha2;
    public string $num;
    public array $searchType=[];

    public function mount()
    {
        $this->searchType["Dateless Style"]=["count"=>0,"dataSet"=>[],"page"=>1,"pages"=>0,"perPage"=>20];
        $this->searchType["Prefix Style"]=["count"=>0,"dataSet"=>[],"page"=>1,"pages"=>0,"perPage"=>20];
        $this->searchType["Current Style"]=["count"=>0,"dataSet"=>[],"page"=>1,"pages"=>0,"perPage"=>20];
    }
    private function prepareVrn(\stdClass $data): \stdClass
    {
        $vrn = new \stdClass;
        if ($data->dvlatype === "IRISH") {
            $vrn->mark = trim($data->alpha1 . " " . $data->numeric1 . " " . $data->alpha2);
        }
        else {
            $vrn->mark = $data->alpha1 . $data->numeric1 . " " . $data->alpha2;
        }
        $helper = new Plates;
        $vrn->basePrice = $helper->getCustomisedPrice($data->price);

        $vrn->enquiryOnly = $data->enquiryOnly??false;
        $vrn->vat = round($vrn->basePrice*(getenv("VATRATE")/100),2);
        $vrn->dot = 80;
        $vrn->total_price = $vrn->basePrice + $vrn->vat + $vrn->dot;
        $vrn->url="/dvla-number-plates/".str_replace(" ","-",strtolower($vrn->mark));
        return $vrn;
    }
    public function setPage($type,$page){
        $this->searchType[$type]["page"]=$page;
    }
    public function setNavigation($type): array
    {
        $page=$this->searchType[$type]["page"];
        $pages=$this->searchType[$type]["pages"];
        $navigation=[];
        $start=$page-2;
        $end=$page+2;
        if ($start<0) {
            $end += abs($start)+1;
            $start=1;
        }
        if ($end > $pages) $end=$pages;
        if ($start !== 1) $navigation[]=1; // ensure first page is accessible as long as we arent on first page
        while($start<=$end){
            $navigation[]=$start;
            $start++;
        }

        if ($end !== $pages) $navigation[]=$pages;
        return $navigation;
    }

    public function getData()
    {
        $perPage=20;
        DVLAClient::Init("ACEREG87");
        $this->searchType["Prefix Style"]["perPage"]=$perPage;
        $this->searchType["Current Style"]["perPage"]=$perPage;
        $this->searchType["Dateless Style"]["perPage"]=$perPage;
        $this->searchType["Prefix Style"]["dataSet"]=[];
        $this->searchType["Current Style"]["dataSet"]=[];
        $this->searchType["Dateless Style"]["dataSet"]=[];


        if ($this->type==="prefix" || $this->type==="all") {
            DVLAClient::setPageControl($this->searchType["Prefix Style"]["page"],$perPage);
            $dvlaPrefixActive=false;
            $summary=false;
            if ($this->alpha2 && ($this->alpha1 || $this->num)) {
                $lets = $this->alpha2;
                if (strlen($lets)==2) $lets .="_";
                if (strlen($lets)==1) $lets .="__";

                DVLAClient::setFilter($this->alpha1, $this->num, $lets);
                $dvlaPrefixActive=true;
            }
            if (!$dvlaPrefixActive && $this->alpha2) {
                DVLAClient::setFilter("","",$this->alpha2);
                $summary=true;
                $dvlaPrefixActive=true;
            }

            $response = DVLAClient::Retrieve("prefix", "alpha2");

            if ($response && DVLAClient::Successful()) {
                if (property_exists($response, "totalAvailable")) {
                    $this->searchType["Prefix Style"]["count"] = $response->totalAvailable;
                    if ($response->totalAvailable > 0){
                        $this->searchType["Prefix Style"]["pages"] = ceil($response->totalAvailable/$perPage);
                        if ($summary && $response->totalAvailable > $perPage) {
                            $this->searchType["Prefix Style"]["note"] = "Our database has around 8 million prefix registrations. " .
                                " Searching just for " . $this->alpha2 . " will bring up too many matches so here is a sample. ";
                        }
                        foreach($response->data as $data){
                            $this->searchType["Prefix Style"]["dataSet"][]=$this->prepareVrn($data);
                        }
                    }
                }
                if (property_exists($response, "error")) {
                    $this->searchType["Prefix Style"]["note"] = $response->error;
                }
                // if there is not enough data passed in or no matches found we dont need to handle the error as
                // without any count, we wont show anythign on scren
                if (property_exists($response, "code") && $response->code === "NOTENOUGH") {
                } elseif (property_exists($response, "code") && $response->code === "NOMATCHES") {
                    // dont need to do anything if we just get a nomatches response
                } elseif (property_exists($response, "code") && $response->code === "NOVALID") {
                    $this->searchType["Prefix Style"]["note"]="There was a problem checking the unissued numbers. Please get in touch";
                    // dont need to do anything if we just get a nomatches response
                }
                $this->searchType["Prefix Style"]["navigation"]=$this->setNavigation("Prefix Style");
            }
        }


        if ($this->type==="current" || $this->type==="all") {
            DVLAClient::setPageControl($this->searchType["Current Style"]["page"],$perPage);
            $dvlaCurrentActive=false;
            $summary=false;
            if ($this->alpha2 && ($this->alpha1 || $this->num)) {
                DVLAClient::setFilter($this->alpha1, $this->num, $this->alpha2);
                $dvlaCurrentActive=true;
            }
            if (!$dvlaCurrentActive && $this->alpha2) {
                DVLAClient::setFilter("","",$this->alpha2);
                $summary=true;
                $dvlaCurrentActive=true;
            }

            if ($dvlaCurrentActive){
                $response = DVLAClient::Retrieve("current", "alpha2");
                if ($response && DVLAClient::Successful()) {
                    if (property_exists($response, "totalAvailable")) {
                        $this->searchType["Current Style"]["count"] = $response->totalAvailable;
                        if ($response->totalAvailable > 0){
                            $this->searchType["Current Style"]["pages"] = ceil($response->totalAvailable/$perPage);
                            if ($summary && $response->totalAvailable > $perPage) {
                                $this->searchType["Current Style"]["note"] = "Our database has around 50 million current style registrations. " .
                                    " Searching just for " . $this->alpha2 . " will bring up too many matches so here is a sample. ";
                            }
                            foreach($response->data as $data){
                                $this->searchType["Current Style"]["dataSet"][]=$this->prepareVrn($data);
                            }
                        }
                    }
                    if (property_exists($response, "error")) {
                        $this->searchType["Current Style"]["note"] = $response->error;
                    }
                    // if there is not enough data passed in or no matches found we dont need to handle the error as
                    // without any count, we wont show anything on screen
                    if (property_exists($response, "code") && $response->code === "NOTENOUGH") {
                    } elseif (property_exists($response, "code") && $response->code === "NOMATCHES") {
                        // dont need to do anything if we just get a nomatches response
                    } elseif (property_exists($response, "code") && $response->code === "NOVALID") {
                        $this->searchType["Current Style"]["note"]="There was a problem checking the unissued numbers. Please get in touch";
                        // dont need to do anything if we just get a nomatches response
                    }
                    $this->searchType["Current Style"]["navigation"]=$this->setNavigation("Current Style");
                }

            }

        }

        if ($this->type==="dateless" || $this->type==="all") {
            DVLAClient::setPageControl($this->searchType["Dateless Style"]["page"],$perPage);
            $dvlaDatelessActive=false;
            $summary=false;
            if ($this->alpha2 || $this->num) {
                if ($this->type==="dateless")
                    DVLAClient::setFilter($this->alpha2, $this->num, $this->alpha1); // dvla irish have letters at start
                else
                    DVLAClient::setFilter($this->alpha1, $this->num, $this->alpha2);
                $dvlaDatelessActive=true;
            }

            if ($dvlaDatelessActive){
                $response = DVLAClient::Retrieve("irish", "alpha2");

                if ($response && DVLAClient::Successful()) {
                    if (property_exists($response, "totalAvailable")) {
                        $this->searchType["Dateless Style"]["count"] = $response->totalAvailable;
                        if ($response->totalAvailable > 0){
                            $this->searchType["Dateless Style"]["pages"] = ceil($response->totalAvailable/$perPage);
                            foreach($response->data as $data){
                                $this->searchType["Dateless Style"]["dataSet"][]=$this->prepareVrn($data);
                            }
                        }
                    }
                    if (property_exists($response, "error")) {
                        $this->searchType["Dateless Style"]["note"] = $response->error;
                    }
                    // if there is not enough data passed in or no matches found we dont need to handle the error as
                    // without any count, we wont show anythign on scren
                    if (property_exists($response, "code") && $response->code === "NOTENOUGH") {
                    } elseif (property_exists($response, "code") && $response->code === "NOMATCHES") {
                        // dont need to do anything if we just get a nomatches response
                    } elseif (property_exists($response, "code") && $response->code === "NOVALID") {
                        $this->searchType["Dateless Style"]["note"]="There was a problem checking the unissued numbers. Please get in touch";
                        // dont need to do anything if we just get a nomatches response
                    }
                    $this->searchType["Dateless Style"]["navigation"]=$this->setNavigation("Dateless Style");
                }
            }
        }
    }

    public function render()
    {
        $this->getData();
       // if ($this->searchType["Current Style"]["page"]===2)        dump($this->searchType["Current Style"]["dataSet"]);
        return view('livewire.dvla');
    }
}
